@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: ReadWriteLocker.h 2381 2007-02-05 16:19:16Z svn $
+ * $Id: ReadWriteLocker.h 2459 2007-06-29 16:38:34Z svn $
  *****************************************************************************/
 
 #ifndef READ_WRITE_LOCKER_H
@@ -39,8 +39,10 @@
 
 #include "AtomicWrapper.h"
 #include "apr_thread_proc.h"
+#include "apr_time.h"
 
 #include "Uncopyable.h"
+#include "Macro.h"
 
 /**
  * @brief 読み書きロックの設定を集めたクラス．
@@ -48,22 +50,25 @@
 class ReadWriteLocker: public Uncopyable
 {
 public:
-    static const apr_uint32_t NOT_LOCKED        = 0x000000;
+    static const apr_uint32_t NOT_LOCKED        = 0x0000;
 
 protected:
     /**
      * コンストラクタです．
      *
-     * @param[out] read_count 読み出し中のスレッドの数
+     * @param[out] lock ロック変数
      */
-    ReadWriteLocker(apr_atomic_t *read_count)
-      : read_count_(read_count)
+    ReadWriteLocker(apr_atomic_t *lock)
+     : lock_(lock),
+       start_age_(0xFFFF),
+       start_time_(apr_time_now()),
+       status_(0)
     {
 
     };
 
     ReadWriteLocker(const ReadWriteLocker& locker)
-      : read_count_(locker.read_count_)
+      : lock_(locker.lock_)
     {
 
     };
@@ -84,15 +89,45 @@ protected:
 #endif
 #endif
     };
+    /**
+     * タイムアウト処理すべきかどうか判断します．
+     *
+     * @param[in] status ロックの状態
+     */
+    bool is_timeout(apr_uint32_t status);
 
     // 注意: ReadWriteLocker 型のポインタ経由で使用されることはないので，
     // デストラクタは virtual にしない．
 
-    // apr_atomic_t に 24bit の制限があるので (APR 0.9.*)
-    static const apr_uint32_t WRITE_LOCKED      = 0xFFFFFF;
+    static const apr_uint32_t WRITE_LOCKED      = 0xFFFF;
+
+    static const apr_size_t AGE_MASK            = 0xFF;
+    static const apr_size_t AGE_SHIFT           = 16;
+    static const apr_size_t READ_COUNT_MASK     = 0xFFFF;
+    static const apr_size_t READ_COUNT_SHIFT    = 0;
+
+    static const apr_size_t TIMEOUT_CHECK_MASK  = 0x1F;
+    static const apr_size_t TIMEOUT_SEC         = 30;
+
+    static apr_uint32_t get_age(apr_uint32_t status)
+    {
+        return (status >> AGE_SHIFT) & AGE_MASK;
+    };
+    static apr_uint32_t get_read_count(apr_uint32_t status)
+    {
+        return (status >> READ_COUNT_SHIFT) & READ_COUNT_MASK;
+    };
+    static apr_uint32_t make_status(apr_uint32_t age, apr_uint32_t read_count)
+    {
+        return ((age & AGE_MASK) << AGE_SHIFT) |
+            ((read_count & READ_COUNT_MASK) << READ_COUNT_SHIFT);
+    };
 
     /** 読み出し中のスレッドの数へのポインタ */
-    apr_atomic_t *read_count_;
+    apr_atomic_t *lock_;
+    apr_uint32_t start_age_;
+    apr_time_t start_time_;
+    apr_uint32_t status_;
 };
 
 #endif
