@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: PostFlowController.cpp 2313 2006-12-24 22:56:51Z svn $
+ * $Id: PostFlowController.cpp 2381 2007-02-05 16:19:16Z svn $
  *****************************************************************************/
 
 #include "Environment.h"
@@ -34,7 +34,7 @@
 #include "WriteLocker.h"
 #include "SourceInfo.h"
 
-SOURCE_INFO_ADD("$Id: PostFlowController.cpp 2313 2006-12-24 22:56:51Z svn $");
+SOURCE_INFO_ADD("$Id: PostFlowController.cpp 2381 2007-02-05 16:19:16Z svn $");
 
 /******************************************************************************
  * public メソッド
@@ -44,7 +44,7 @@ PostFlowController::PostFlowController(apr_size_t min_post_interval_sec)
     poster_top_(0),
     poster_tail_(0)
 {
-    (void)apr_atomic_set(&lock_, ReadWriteLocker::NOT_LOCKER);
+    (void)apr_atomic_set(&lock_, ReadWriteLocker::NOT_LOCKED);
 }
 
 bool PostFlowController::can_post(apr_sockaddr_t *sockaddr)
