@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: uploader_handler.cpp 2422 2007-05-13 14:50:22Z svn $
+ * $Id: uploader_handler.cpp 2438 2007-06-26 13:38:49Z svn $
  *****************************************************************************/
 
 #ifndef TEMPLATE_INSTANTIATION
@@ -49,6 +49,7 @@
 #include "ThumbnailIterator.h"
 #include "ThumbnailVariableCreator.h"
 #include "PostProgressList.h"
+#include "DownloadFlowController.h"
 #include "CharCodeConverter.h"
 #include "Message.h"
 #include "Macro.h"
@@ -56,7 +57,7 @@
 #include "SourceInfo.h"
 
 #ifndef TEMPLATE_INSTANTIATION
-SOURCE_INFO_ADD("$Id: uploader_handler.cpp 2422 2007-05-13 14:50:22Z svn $");
+SOURCE_INFO_ADD("$Id: uploader_handler.cpp 2438 2007-06-26 13:38:49Z svn $");
 #endif
 
 #define IS_REQUEST_COMMAND(arg, command)                        \
@@ -91,6 +92,8 @@ static const char INFO_COMMAND[]                = "info";
 
 static const char PAGE_CONTENT_TYPE[]           = "text/html; charset=EUC-JP";
 
+static const apr_size_t DOWNLOAD_BUSY_SLEEP_SEC = 5;
+
 template<class Response>
 static int redirect(typename Response::Handle *r, typename Response::Writer& o,
                     const char *url)
@@ -688,83 +691,78 @@ static int download(typename Response::Handle *r, UploaderConfig *config,
             }
         }
 
-        if (!can_download(config, Response::get_remote_addr(r))) {
+        DownloadSession download_session
+            (config->get_download_flow_controller(),
+             Response::get_remote_addr(r));
+        if (!is_admin_mode && !download_session.is_valid()) {
+            apr_sleep(apr_time_from_sec(DOWNLOAD_BUSY_SLEEP_SEC));
             THROW(MESSAGE_DOWNLOAD_SESSION_LIMIT_EXCEEDED);
         }
 
-        try {
-            item_file = config->get_item_manager()
-                ->get_item_file(r->pool, item_id,
-                                Response::Writer::is_sendfile_enabled(r));
+        item_file = config->get_item_manager()
+            ->get_item_file(r->pool, item_id,
+                            Response::Writer::is_sendfile_enabled(r));
 
 #ifdef RESPONSE_TYPE_APACHE
-            {
-                const char *item_path;
-                apr_file_name_get(&item_path, item_file);
-                r->filename = const_cast<char *>(item_path);
-                r->finfo.filetype = APR_REG;
-                r->finfo.size = uitem->get_file_size();
-            }
-            Response::set_content_type(r);
+        {
+            const char *item_path;
+            apr_file_name_get(&item_path, item_file);
+            r->filename = const_cast<char *>(item_path);
+            r->finfo.filetype = APR_REG;
+            r->finfo.size = uitem->get_file_size();
+        }
+        Response::set_content_type(r);
 #else
-            Response::set_content_type(r, uitem->get_file_mime());
+        Response::set_content_type(r, uitem->get_file_mime());
 #endif
 
 #ifndef DEBUG
-            Response::set_modified_time(r, uitem->get_mtime());
-            Response::set_last_modified(r, uitem->get_mtime());
+        Response::set_modified_time(r, uitem->get_mtime());
+        Response::set_last_modified(r, uitem->get_mtime());
 
-            int status;
-            if ((status = Response::is_meets_condition(r)) != OK) {
-                finish_download(config, Response::get_remote_addr(r));
-                return status;
-            }
+        int status;
+        if ((status = Response::is_meets_condition(r)) != OK) {
+            return status;
+        }
 #endif
-            if (Response::is_head_method(r)) {
-                finish_download(config, Response::get_remote_addr(r));
-                return OK;
-            }
+        if (Response::is_head_method(r)) {
+            return OK;
+        }
 
-            Response::set_accept_ranges(r);
+        Response::set_accept_ranges(r);
 
-            dispos_encoding = get_dispos_encoding(Response::get_user_agent(r));
+        dispos_encoding = get_dispos_encoding(Response::get_user_agent(r));
 
 #ifdef NUMNAME
-            file_name = apr_psprintf(r->pool, "%s%" APR_SIZE_T_FMT ".%s",
-                                     NUMNAME, uitem->get_index(),
-                                     uitem->get_file_ext());
+        file_name = apr_psprintf(r->pool, "%s%" APR_SIZE_T_FMT ".%s",
+                                 NUMNAME, uitem->get_index(),
+                                 uitem->get_file_ext());
 #else
 #ifdef CONV_CHAR_CODE
-            file_name = CharCodeConverter::convert
-                (r->pool, uitem->get_file_name(),
-                 CharCodeConverter::DEFAULT_CODE, dispos_encoding);
+        file_name = CharCodeConverter::convert
+            (r->pool, uitem->get_file_name(),
+             CharCodeConverter::DEFAULT_CODE, dispos_encoding);
 #else
-            file_name = uitem->get_file_name();
+        file_name = uitem->get_file_name();
 #endif
 #endif
 
-            // 表示モード or ダウンロードモード
-            open_mode = (strlen(get_word(r->pool, &arg,
-                                         ARG_SEPARATE_STR[0])) == 0)
-                ? "inline" : "attachment";
-            // Content-Disposition
-            Response::set_content_disposition
-                (r,
-                 apr_psprintf(r->pool, "%s; filename=\"%s\"", open_mode,
-                              file_name));
+        // 表示モード or ダウンロードモード
+        open_mode = (strlen(get_word(r->pool, &arg,
+                                     ARG_SEPARATE_STR[0])) == 0)
+            ? "inline" : "attachment";
+        // Content-Disposition
+        Response::set_content_disposition
+            (r,
+             apr_psprintf(r->pool, "%s; filename=\"%s\"", open_mode,
+                          file_name));
 
-            Response::set_env(r, "uploader_download", "");
-            Response::set_env(r, "uploader_item_id",
-                              apr_psprintf(r->pool, "%u", item_id));
+        Response::set_env(r, "uploader_download", "");
+        Response::set_env(r, "uploader_item_id",
+                          apr_psprintf(r->pool, "%u", item_id));
 
-            // better to use ap_internal_fast_redirect for Apache
-            Response::Writer::sendfile(r, item_file, uitem->get_file_size());
-
-            finish_download(config, Response::get_remote_addr(r));
-        } catch(const char *) {
-            finish_download(config, Response::get_remote_addr(r));
-            throw;
-        }
+        // better to use ap_internal_fast_redirect for Apache
+        Response::Writer::sendfile(r, item_file, uitem->get_file_size());
     } catch(const char *message) {
         return error<Response>(r, config, message);
     }
