@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: uploader_util.cpp 2220 2006-12-10 06:10:27Z svn $
+ * $Id: uploader_util.cpp 2438 2007-06-26 13:38:49Z svn $
  *****************************************************************************/
 
 #include "Environment.h"
@@ -40,7 +40,7 @@
 #include "Auxiliary.h"
 #include "SourceInfo.h"
 
-SOURCE_INFO_ADD("$Id: uploader_util.cpp 2220 2006-12-10 06:10:27Z svn $");
+SOURCE_INFO_ADD("$Id: uploader_util.cpp 2438 2007-06-26 13:38:49Z svn $");
 
 const char *get_word(apr_pool_t *pool, const char **input,
                      const char delimiter)
@@ -127,16 +127,6 @@ void regist_post(UploaderConfig *config, apr_sockaddr_t *ip_address)
     config->get_post_flow_controller()->regist_post(ip_address);
 }
 
-bool can_download(UploaderConfig *config, apr_sockaddr_t *ip_address)
-{
-    return config->get_download_flow_controller()->add_session(ip_address);
-}
-
-void finish_download(UploaderConfig *config, apr_sockaddr_t *ip_address)
-{
-    config->get_download_flow_controller()->remove_session(ip_address);
-}
-
 // Local Variables:
 // mode: c++
 // coding: utf-8-dos
