@@ -108,7 +108,13 @@ sub compile_module {
     $exp->send('make -f GNUmakefile.win32 clean 2>&1 | nkf -e' . "\n");
     $exp->expect(MAKE_TIMEOUT_SEC, 'src>');
 
-    $exp->send('make -f GNUmakefile.win32 2>&1 | nkf -e' . "\n");
+    $exp->send('make -f GNUmakefile.win32 mod_uploader-ap2.0.so 2>&1 | nkf -e' . "\n");
+    $exp->expect(MAKE_TIMEOUT_SEC, 'src>');
+
+    $exp->send('make -f GNUmakefile.win32 clean-obj 2>&1 | nkf -e' . "\n");
+    $exp->expect(MAKE_TIMEOUT_SEC, 'src>');
+
+    $exp->send('make -f GNUmakefile.win32 mod_uploader-ap2.2.so 2>&1 | nkf -e' . "\n");
     $exp->expect(MAKE_TIMEOUT_SEC, 'src>');
 }
 
